@@ -30,7 +30,11 @@ sub configure_hook {
     $self->{server}->{host}   = INetSim::Config::getConfigParameter("Default_BindAddress"); # bind to address
     $self->{server}->{proto}  = 'tcp';                                                      # TCP protocol
     $self->{server}->{user}   = INetSim::Config::getConfigParameter("Default_RunAsUser");   # user to run as
+    $self->{server}->{user}   =~ /\A(.*)\z/; # evil untaint!
+    $self->{server}->{user}   = $1;
     $self->{server}->{group}  = INetSim::Config::getConfigParameter("Default_RunAsGroup");  # group to run as
+    $self->{server}->{group}  =~ /\A(.*)\z/; # evil untaint!
+    $self->{server}->{group}  = $1;
     $self->{server}->{setsid} = 0;                                                          # do not daemonize
     $self->{server}->{no_client_stdout} = 1;                                                # do not attach client to STDOUT
     $self->{server}->{log_level} = 0;                                                       # do not log anything
@@ -126,17 +130,17 @@ sub configure_hook {
         exit 1;
     }
 
-    $gid = getgrnam(INetSim::Config::getConfigParameter("Default_RunAsGroup"));
+    $gid = getgrnam($self->{server}->{group});
     if (! defined $gid) {
-        INetSim::Log::MainLog("Warning: Unable to get GID for group 'inetsim'", $self->{servicename});
+        INetSim::Log::MainLog("Warning: Unable to get GID for group '$self->{server}->{group}'", $self->{servicename});
     }
     chown -1, $gid, $self->{upload_dir};
     ($dev, $inode, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat $self->{upload_dir};
 
-    # check for group owner 'inetsim'
+    # check group owner
     $grpname = getgrgid $gid;
-    if ($grpname ne INetSim::Config::getConfigParameter("Default_RunAsGroup")) {
-        INetSim::Log::MainLog("Warning: Group owner of Upload directory '$self->{upload_dir}' is not 'inetsim' but '$grpname'", $self->{servicename});
+    if ($grpname ne $self->{server}->{group}) {
+        INetSim::Log::MainLog("Warning: Group owner of Upload directory '$self->{upload_dir}' is not '$self->{server}->{group}' but '$grpname'", $self->{servicename});
     }
     # check for group r/w permissions
     if ((($mode & 0060) >> 3) != 6) {
@@ -203,142 +207,139 @@ sub process_request {
         $self->slog_("connect");
         $self->slog_("info: Error setting up SSL:  $self->{last_ssl_error}");
         $self->slog_("disconnect");
+        $self->slog_("stat: 0");
+        return;
     }
-    elsif ($self->{server}->{numchilds} >= $self->{maxchilds}) {
+    if ($self->{server}->{numchilds} >= $self->{maxchilds}) {
         $self->send_("421 Maximum number of connections ($self->{maxchilds}) exceeded.");
+        $self->slog_("stat: 0");
+        return;
     }
-    else {
-        my $line = "";
-        eval {
-            local $SIG{'ALRM'} = sub { die "TIMEOUT" };
+    my $line = "";
+    eval {
+        local $SIG{'ALRM'} = sub { die "TIMEOUT" };
+        alarm($self->{timeout});
+        $self->slog_("connect");
+        ### Server Greeting
+        $self->send_("220 $self->{banner}");
+        ### Now wait for command
+        while ($line = <$client>){
+            chomp($line);
+            $line =~ s/\r\z//g;
+            $line =~ s/[\r\n]+//g;
             alarm($self->{timeout});
-            $self->slog_("connect");
-            ### Server Greeting
-            $self->send_("220 $self->{banner}");
-            ### Now wait for command
-            while ($line = <$client>){
-                chomp($line);
-                $line =~ s/\r\z//g;
-                $line =~ s/[\r\n]+//g;
-                alarm($self->{timeout});
-                $self->slog_("recv: $line");
-                if (defined ($line) && $line =~ /\AQUIT(|([\s]+)(.*))\z/i) {
-                    $self->QUIT;
-                    last;
-                }
-                elsif (defined ($line) && $line =~ /\AUSER[\s\t]+(.*)\z/i) {
-                    $self->USER($1);
-                }
-                elsif (defined ($line) && $line =~ /\APASS[\s\t]+(.*)\z/i) {
-                    $self->PASS($1);
-                    if ($self->{authenticated}) {
-                        $stat_success = 1;
-                        $credentials = "$self->{username}:$self->{password}";
-                    }
-                }
-                elsif (defined ($line) && $line =~ /\ASYST(|([\s\t]+)(.*))\z/i) {
-                    $self->SYST;
-                }
-                elsif (defined ($line) && $line =~ /\ALIST(|[\s\t]+(.*))\z/i) {
-                    $self->LIST($2);
-                }
-                elsif (defined ($line) && $line =~ /\ANLST(|[\s\t]+(.*))\z/i) {
-                    $self->NLST($2);
-                }
-                elsif (defined ($line) && $line =~ /\APORT[\s\t]+([\d\,]+)\z/i) {
-                    $self->PORT($1);
-                }
-                elsif (defined ($line) && $line =~ /\ACWD[\s\t]+(.*)\z/i) {
-                    $self->CWD($1);
-                }
-                elsif (defined ($line) && $line =~ /\ACDUP(|([\s\t]+)(.*))\z/i) {
-                    $self->CDUP;
-                }
-                elsif (defined ($line) && $line =~ /\ARMD[\s\t]+(.*)\z/i) {
-                    $self->RMD($1);
-                }
-                elsif (defined ($line) && $line =~ /\AMKD[\s\t]+(.*)\z/i) {
-                    $self->MKD($1);
-                }
-                elsif (defined ($line) && $line =~ /\APWD(|([\s\t]+)(.*))\z/i) {
-                    $self->PWD;
-                }
-#                elsif (defined ($line) && $line =~ /\AABOR(|([\s\t]+)(.*))\z/i) {
-#                    $self->ABOR;
-#                }
-                elsif (defined ($line) && $line =~ /\ADELE[\s\t]+(.*)\z/i) {
-                    $self->DELE($1);
-                }
-                elsif (defined ($line) && $line =~ /\ATYPE[\s\t]+(\w(\s[\w\d])?)\z/i) {
-                    $self->TYPE($1);
-                }
-                elsif (defined ($line) && $line =~ /\ASTRU[\s\t]+(\w)\z/i) {
-                    $self->STRU($1);
-                }
-                elsif (defined ($line) && $line =~ /\AMODE[\s\t]+(\w)\z/i) {
-                    $self->MODE($1);
-                }
-                elsif (defined ($line) && $line =~ /\ANOOP(|([\s\t]+)(.*))\z/i) {
-                    $self->NOOP;
-                }
-                elsif (defined ($line) && $line =~ /\AHELP(|[\s\t]+(.*))\z/i) {
-                    $self->HELP($2);
-                }
-                elsif (defined ($line) && $line =~ /\ASTAT(|[\s\t]+(.*))\z/i) {
-                    $self->STAT($2);
-                }
-                elsif (defined ($line) && $line =~ /\ASTOR[\s\t]+(.*)\z/i) {
-                    $self->STOR($1);
-                }
-#                elsif (defined ($line) && $line =~ /\ASTOU(|[\s\t]+(.*))\z/i) {
-#                    $self->STOU($2);
-#                }
-                elsif (defined ($line) && $line =~ /\ARETR[\s\t]+(.*)\z/i) {
-                    $self->RETR($1);
-                }
-#                elsif (defined ($line) && $line =~ /\AREIN(|([\s\t]+)(.*))\z/i) {
-#                    $self->REIN;
-#                }
-#                elsif (defined ($line) && $line =~ /\AACCT(|[\s\t]+(.*))\z/i) {
-#                    $self->ACCT($2);
-#                }
-#                elsif (defined ($line) && $line =~ /\AREST(|[\s\t]+(.*))\z/i) {
-#                    $self->REST($2);
-#                }
-#                elsif (defined ($line) && $line =~ /\AAPPE(|[\s\t]+(.*))\z/i) {
-#                    $self->APPE($2);
-#                }
-#                elsif (defined ($line) && $line =~ /\ARNFR(|[\s\t]+(.*))\z/i) {
-#                    $self->RNFR($2);
-#                }
-#                elsif (defined ($line) && $line =~ /\ARNTO(|[\s\t]+(.*))\z/i) {
-#                    $self->RNTO($2);
-#                }
-#                elsif (defined ($line) && $line =~ /\ASITE(|[\s\t]+(.*))\z/i) {
-#                    $self->SITE($2);
-#                }
-                else {
-                    $self->send_("500 Unknown command.");
-                }
-                alarm($self->{timeout});
+            $self->slog_("recv: $line");
+            if (defined ($line) && $line =~ /\AQUIT(|([\s]+)(.*))\z/i) {
+                $self->QUIT;
+                last;
             }
-        };
-        alarm(0);
-        if ($@ =~ /TIMEOUT/) {
-            $self->send_("421 Error: timeout exceeded");
-            $self->slog_("disconnect (timeout)");
+            elsif (defined ($line) && $line =~ /\AUSER[\s\t]+(.*)\z/i) {
+                $self->USER($1);
+            }
+            elsif (defined ($line) && $line =~ /\APASS[\s\t]+(.*)\z/i) {
+                $self->PASS($1);
+                if ($self->{authenticated}) {
+                    $stat_success = 1;
+                    $credentials = "$self->{username}:$self->{password}";
+                }
+            }
+            elsif (defined ($line) && $line =~ /\ASYST(|([\s\t]+)(.*))\z/i) {
+                $self->SYST;
+            }
+            elsif (defined ($line) && $line =~ /\ALIST(|[\s\t]+(.*))\z/i) {
+                $self->LIST($2);
+            }
+            elsif (defined ($line) && $line =~ /\ANLST(|[\s\t]+(.*))\z/i) {
+                $self->NLST($2);
+            }
+            elsif (defined ($line) && $line =~ /\APORT[\s\t]+([\d\,]+)\z/i) {
+                $self->PORT($1);
+            }
+            elsif (defined ($line) && $line =~ /\ACWD[\s\t]+(.*)\z/i) {
+                $self->CWD($1);
+            }
+            elsif (defined ($line) && $line =~ /\ACDUP(|([\s\t]+)(.*))\z/i) {
+                $self->CDUP;
+            }
+            elsif (defined ($line) && $line =~ /\ARMD[\s\t]+(.*)\z/i) {
+                $self->RMD($1);
+            }
+            elsif (defined ($line) && $line =~ /\AMKD[\s\t]+(.*)\z/i) {
+                $self->MKD($1);
+            }
+            elsif (defined ($line) && $line =~ /\APWD(|([\s\t]+)(.*))\z/i) {
+                $self->PWD;
+            }
+#            elsif (defined ($line) && $line =~ /\AABOR(|([\s\t]+)(.*))\z/i) {
+#                $self->ABOR;
+#            }
+            elsif (defined ($line) && $line =~ /\ADELE[\s\t]+(.*)\z/i) {
+                $self->DELE($1);
+            }
+            elsif (defined ($line) && $line =~ /\ATYPE[\s\t]+(\w(\s[\w\d])?)\z/i) {
+                $self->TYPE($1);
+            }
+            elsif (defined ($line) && $line =~ /\ASTRU[\s\t]+(\w)\z/i) {
+                $self->STRU($1);
+            }
+            elsif (defined ($line) && $line =~ /\AMODE[\s\t]+(\w)\z/i) {
+                $self->MODE($1);
+            }
+            elsif (defined ($line) && $line =~ /\ANOOP(|([\s\t]+)(.*))\z/i) {
+                $self->NOOP;
+            }
+            elsif (defined ($line) && $line =~ /\AHELP(|[\s\t]+(.*))\z/i) {
+                $self->HELP($2);
+            }
+            elsif (defined ($line) && $line =~ /\ASTAT(|[\s\t]+(.*))\z/i) {
+                $self->STAT($2);
+            }
+            elsif (defined ($line) && $line =~ /\ASTOR[\s\t]+(.*)\z/i) {
+                $self->STOR($1);
+            }
+#            elsif (defined ($line) && $line =~ /\ASTOU(|[\s\t]+(.*))\z/i) {
+#                $self->STOU($2);
+#            }
+            elsif (defined ($line) && $line =~ /\ARETR[\s\t]+(.*)\z/i) {
+                $self->RETR($1);
+            }
+#            elsif (defined ($line) && $line =~ /\AREIN(|([\s\t]+)(.*))\z/i) {
+#                $self->REIN;
+#            }
+#            elsif (defined ($line) && $line =~ /\AACCT(|[\s\t]+(.*))\z/i) {
+#                $self->ACCT($2);
+#            }
+#            elsif (defined ($line) && $line =~ /\AREST(|[\s\t]+(.*))\z/i) {
+#                $self->REST($2);
+#            }
+#            elsif (defined ($line) && $line =~ /\AAPPE(|[\s\t]+(.*))\z/i) {
+#                $self->APPE($2);
+#            }
+#            elsif (defined ($line) && $line =~ /\ARNFR(|[\s\t]+(.*))\z/i) {
+#                $self->RNFR($2);
+#            }
+#            elsif (defined ($line) && $line =~ /\ARNTO(|[\s\t]+(.*))\z/i) {
+#                $self->RNTO($2);
+#            }
+#            elsif (defined ($line) && $line =~ /\ASITE(|[\s\t]+(.*))\z/i) {
+#                $self->SITE($2);
+#            }
+            else {
+                $self->send_("500 Unknown command.");
+            }
+            alarm($self->{timeout});
         }
-        else {
-            $self->slog_("disconnect");
-        }
-        $self->_save_vfs_changes;
-    }
-    if ($stat_success == 1) {
-        $self->slog_("stat: $stat_success created=$self->{count_created} deleted=$self->{count_deleted} retrieved=$self->{count_retrieved} creds=$credentials");
+    };
+    alarm(0);
+    if ($@ =~ /TIMEOUT/) {
+        $self->send_("421 Error: timeout exceeded");
+        $self->slog_("disconnect (timeout)");
     }
     else {
-        $self->slog_("stat: $stat_success");
+        $self->slog_("disconnect");
     }
+    $self->_save_vfs_changes;
+    $self->slog_("stat: $stat_success" . (($stat_success == 1) ? " created=$self->{count_created} deleted=$self->{count_deleted} retrieved=$self->{count_retrieved} creds=$credentials" : ""));
 }
 
 
