@@ -55,7 +55,11 @@ sub configure_hook {
     $self->{server}->{host}   = INetSim::Config::getConfigParameter("Default_BindAddress"); # bind to address
     $self->{server}->{proto}  = 'tcp';                                      # TCP protocol
     $self->{server}->{user}   = INetSim::Config::getConfigParameter("Default_RunAsUser");       # user to run as
+    $self->{server}->{user}   =~ /\A(.*)\z/; # evil untaint!
+    $self->{server}->{user}   = $1;
     $self->{server}->{group}  = INetSim::Config::getConfigParameter("Default_RunAsGroup");    # group to run as
+    $self->{server}->{group}  =~ /\A(.*)\z/; # evil untaint!
+    $self->{server}->{group}  = $1;
     $self->{server}->{setsid} = 0;                                   # do not daemonize
     $self->{server}->{no_client_stdout} = 1;                         # do not attach client to STDOUT
     $self->{server}->{log_level} = 0;                                # do not log anything
@@ -147,16 +151,16 @@ sub configure_hook {
     else {
         close DAT;
         chmod 0660, $self->{datfile};
-        $gid = getgrnam(INetSim::Config::getConfigParameter("Default_RunAsGroup"));
+        $gid = getgrnam($self->{server}->{group});
         if (! defined $gid) {
-            INetSim::Log::MainLog("Warning: Unable to get GID for group 'inetsim'", $self->{servicename});
+            INetSim::Log::MainLog("Warning: Unable to get GID for group '$self->{server}->{group}'", $self->{servicename});
         }
         chown -1, $gid, $self->{datfile};
         ($dev, $inode, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat $self->{datfile};
         $grpname = getgrgid $gid;
-        # check for group owner 'inetsim'
-        if ($grpname ne INetSim::Config::getConfigParameter("Default_RunAsGroup")) {
-            INetSim::Log::MainLog("Warning: Group owner of POP3 main datafile '$self->{datfile}' is not 'inetsim' but '$grpname'", $self->{servicename});
+        # check group owner
+        if ($grpname ne $self->{server}->{group}) {
+            INetSim::Log::MainLog("Warning: Group owner of POP3 main datafile '$self->{datfile}' is not '$self->{server}->{group}' but '$grpname'", $self->{servicename});
         }
         # check for group r/w permissions
         if ((($mode & 0060) >> 3) != 6) {
@@ -170,16 +174,16 @@ sub configure_hook {
     else {
         close LCK;
         chmod 0660, $self->{sessionlockfile};
-        $gid = getgrnam("inetsim");
+        $gid = getgrnam($self->{server}->{group});
         if (! defined $gid) {
-            INetSim::Log::MainLog("Warning: Unable to get GID for group 'inetsim'", $self->{servicename});
+            INetSim::Log::MainLog("Warning: Unable to get GID for group '$self->{server}->{group}'", $self->{servicename});
         }
         chown -1, $gid, $self->{sessionlockfile};
         ($dev, $inode, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat $self->{sessionlockfile};
         $grpname = getgrgid $gid;
-        # check for group owner 'inetsim'
-        if ($grpname ne "inetsim") {
-            INetSim::Log::MainLog("Warning: Group owner of POP3 lockfile '$self->{sessionlockfile}' is not 'inetsim' but '$grpname'", $self->{servicename});
+        # check group owner
+        if ($grpname ne $self->{server}->{group}) {
+            INetSim::Log::MainLog("Warning: Group owner of POP3 lockfile '$self->{sessionlockfile}' is not '$self->{server}->{group}' but '$grpname'", $self->{servicename});
         }
         # check for group r/w permissions
         if ((($mode & 0060) >> 3) != 6) {
@@ -193,16 +197,16 @@ sub configure_hook {
     else {
         close SDAT;
         chmod 0660, $self->{sessiondatfile};
-        $gid = getgrnam("inetsim");
+        $gid = getgrnam($self->{server}->{group});
         if (! defined $gid) {
-            INetSim::Log::MainLog("Warning: Unable to get GID for group 'inetsim'", $self->{servicename});
+            INetSim::Log::MainLog("Warning: Unable to get GID for group '$self->{server}->{group}'", $self->{servicename});
         }
         chown -1, $gid, $self->{sessiondatfile};
         ($dev, $inode, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat $self->{sessiondatfile};
         $grpname = getgrgid $gid;
-        # check for group owner 'inetsim'
-        if ($grpname ne "inetsim") {
-            INetSim::Log::MainLog("Warning: Group owner of POP3 session datafile '$self->{sessiondatfile}' is not 'inetsim' but '$grpname'", $self->{servicename});
+        # check group owner
+        if ($grpname ne $self->{server}->{group}) {
+            INetSim::Log::MainLog("Warning: Group owner of POP3 session datafile '$self->{sessiondatfile}' is not '$self->{server}->{group}' but '$grpname'", $self->{servicename});
         }
         # check for group r/w permissions
         if ((($mode & 0060) >> 3) != 6) {
@@ -256,124 +260,121 @@ sub process_request {
         $self->slog_("connect");
         $self->slog_("info: Error setting up SSL:  $self->{last_ssl_error}");
         $self->slog_("disconnect");
+        $self->slog_("stat: 0");
+        return;
     }
-    elsif ($self->{server}->{numchilds} >= $self->{maxchilds}) {
+    if ($self->{server}->{numchilds} >= $self->{maxchilds}) {
         $self->slog_("connect");
         $self->send_("-ERR", "Maximum number of connections ($self->{maxchilds}) exceeded.");
         $self->slog_("disconnect");
+        $self->slog_("stat: 0");
+        return;
+    }
+    $self->slog_("connect");
+    ### Server Greeting
+    if ($self->{enable_apop}) {
+        $self->send_("+OK", "$self->{banner} <$$." . INetSim::FakeTime::get_faketime() . "\@$self->{hostname}>");
     }
     else {
-        $self->slog_("connect");
-        ### Server Greeting
-        if ($self->{enable_apop}) {
-            $self->send_("+OK", "$self->{banner} <$$." . INetSim::FakeTime::get_faketime() . "\@$self->{hostname}>");
-        }
-        else {
-            $self->send_("+OK", "$self->{banner}");
-        }
-        # set variables/flags
-        $self->{last_login} = 0;
-        $self->{lock_error} = 0;
-        $self->{state} = "auth";
-        #
-        srand(time() ^($$ + ($$ <<15)));
+        $self->send_("+OK", "$self->{banner}");
+    }
+    # set variables/flags
+    $self->{last_login} = 0;
+    $self->{lock_error} = 0;
+    $self->{state} = "auth";
+    #
+    srand(time() ^($$ + ($$ <<15)));
 
-        eval {
-            local $SIG{'ALRM'} = sub { die "TIMEOUT" };
+    eval {
+        local $SIG{'ALRM'} = sub { die "TIMEOUT" };
+        alarm($self->{timeout});
+        while ($line = <$client>){
+            ### 1. Waiting for Authorisation - The RFC calls this the Authentication State. Valid commands: USER PASS APOP QUIT
+            ### 2. After 1, switching to Transaction State. Valid commands: STAT LIST RETR DELE NOOP RSET UIDL QUIT
+            $line =~ s/\A[\r\n\s\t]+//g;
+            $line =~ s/[\r\n\s\t]+\z//g;
             alarm($self->{timeout});
-            while ($line = <$client>){
-                ### 1. Waiting for Authorisation - The RFC calls this the Authentication State. Valid commands: USER PASS APOP QUIT
-                ### 2. After 1, switching to Transaction State. Valid commands: STAT LIST RETR DELE NOOP RSET UIDL QUIT
-                $line =~ s/\A[\r\n\s\t]+//g;
-                $line =~ s/[\r\n\s\t]+\z//g;
-                alarm($self->{timeout});
-                $self->slog_("recv: $line");
-                ### Auth via USER/PASS
-                if ($line =~ /\AUSER(|([\s]+)(.*))\z/i && defined $POP3_CAPA{USER}) {
-                    $self->USER($3);
-                }
-                elsif ($line =~ /\APASS(|([\s]+)(.*))\z/i && defined $POP3_CAPA{USER}) {
-                    $self->PASS($3);
-                    if ($self->{close_connection}) {
-                        last;
-                    }
-                }
-                ### Auth via APOP
-                elsif ($line =~ /\AAPOP(|([\s]+)(.*))\z/i && $self->{enable_apop}) {
-                    $self->APOP($3);
-                    if ($self->{close_connection}) {
-                        last;
-                    }
-                }
-                elsif ($line =~ /\AQUIT(|([\s]+)(.*))\z/i) {
-                    $self->QUIT($3);
-                    if ($self->{close_connection}) {
-                        last;
-                    }
-                }
-                elsif ($line =~ /\ASTAT(|([\s]+)(.*))\z/i) {
-                    $self->STAT($3);
-                }
-                elsif ($line =~ /\ALIST(|([\s]+)(.*))\z/i) {
-                    $self->LIST($3);
-                }
-                elsif ($line =~ /\ARETR(|([\s]+)(.*))\z/i) {
-                    $self->RETR($3);
-                }
-                elsif ($line =~ /\ADELE(|([\s]+)(.*))\z/i) {
-                    $self->DELE($3);
-                }
-                elsif ($line =~ /\ANOOP(|([\s]+)(.*))\z/i) {
-                    $self->NOOP($3);
-                }
-                elsif ($line =~ /\ARSET(|([\s]+)(.*))\z/i) {
-                    $self->RSET($3);
-                }
-                elsif ($line =~ /\ATOP(|([\s]+)(.*))\z/i && defined $POP3_CAPA{TOP}) {
-                    $self->TOP($3);
-                }
-                elsif ($line =~ /\AUIDL(|([\s]+)(.*))\z/i && defined $POP3_CAPA{UIDL}) {
-                    $self->UIDL($3);
-                }
-                elsif ($line =~ /\ACAPA(|([\s]+)(.*))\z/i && $self->{capabilities}) {
-                    $self->CAPA($3);
-                }
-                elsif ($line =~ /\AAUTH(|([\s]+)(.*))\z/i && defined $POP3_CAPA{SASL}) {
-                    $self->AUTH($3);
-                    if ($self->{close_connection}) {
-                        last;
-                    }
-                }
-                elsif ($line =~ /\ASTLS(|([\s]+)(.*))\z/i && defined $POP3_CAPA{STLS}) {
-                    $self->STLS($3);
-                    if ($self->{close_connection}) {
-                        last;
-                    }
-                }
-                else {
-                    $self->send_("-ERR", "Unknown command.");
-                }
-                alarm($self->{timeout});
+            $self->slog_("recv: $line");
+            ### Auth via USER/PASS
+            if ($line =~ /\AUSER(|([\s]+)(.*))\z/i && defined $POP3_CAPA{USER}) {
+                $self->USER($3);
             }
-            alarm(0);
-        };
-        if ($@ =~ /TIMEOUT/) {
-            $self->send_("-ERR", "timeout exceeded");
-            $self->slog_("disconnect (timeout)");
+            elsif ($line =~ /\APASS(|([\s]+)(.*))\z/i && defined $POP3_CAPA{USER}) {
+                $self->PASS($3);
+                if ($self->{close_connection}) {
+                    last;
+                }
+            }
+            ### Auth via APOP
+            elsif ($line =~ /\AAPOP(|([\s]+)(.*))\z/i && $self->{enable_apop}) {
+                $self->APOP($3);
+                if ($self->{close_connection}) {
+                    last;
+                }
+            }
+            elsif ($line =~ /\AQUIT(|([\s]+)(.*))\z/i) {
+                $self->QUIT($3);
+                if ($self->{close_connection}) {
+                    last;
+                }
+            }
+            elsif ($line =~ /\ASTAT(|([\s]+)(.*))\z/i) {
+                $self->STAT($3);
+            }
+            elsif ($line =~ /\ALIST(|([\s]+)(.*))\z/i) {
+                $self->LIST($3);
+            }
+            elsif ($line =~ /\ARETR(|([\s]+)(.*))\z/i) {
+                $self->RETR($3);
+            }
+            elsif ($line =~ /\ADELE(|([\s]+)(.*))\z/i) {
+                $self->DELE($3);
+            }
+            elsif ($line =~ /\ANOOP(|([\s]+)(.*))\z/i) {
+                $self->NOOP($3);
+            }
+            elsif ($line =~ /\ARSET(|([\s]+)(.*))\z/i) {
+                $self->RSET($3);
+            }
+            elsif ($line =~ /\ATOP(|([\s]+)(.*))\z/i && defined $POP3_CAPA{TOP}) {
+                $self->TOP($3);
+            }
+            elsif ($line =~ /\AUIDL(|([\s]+)(.*))\z/i && defined $POP3_CAPA{UIDL}) {
+                $self->UIDL($3);
+            }
+            elsif ($line =~ /\ACAPA(|([\s]+)(.*))\z/i && $self->{capabilities}) {
+                $self->CAPA($3);
+            }
+            elsif ($line =~ /\AAUTH(|([\s]+)(.*))\z/i && defined $POP3_CAPA{SASL}) {
+                $self->AUTH($3);
+                if ($self->{close_connection}) {
+                    last;
+                }
+            }
+            elsif ($line =~ /\ASTLS(|([\s]+)(.*))\z/i && defined $POP3_CAPA{STLS}) {
+                $self->STLS($3);
+                if ($self->{close_connection}) {
+                    last;
+                }
+            }
+            else {
+                $self->send_("-ERR", "Unknown command.");
+            }
+            alarm($self->{timeout});
         }
-        else {
-            $self->slog_("disconnect");
-        }
-        if (! $self->{lock_error} && $self->session_lock()){
-            $self->session_lock("unlock");
-        }
-    }
-    if ($status{success}) {
-        $self->slog_("stat: $status{success} retrieved=$status{retrieved} deleted=$status{deleted} auth=$status{auth_type} creds=$status{credentials} tls=$status{tls_used} cipher=$status{tls_cipher}");
+        alarm(0);
+    };
+    if ($@ =~ /TIMEOUT/) {
+        $self->send_("-ERR", "timeout exceeded");
+        $self->slog_("disconnect (timeout)");
     }
     else {
-        $self->slog_("stat: $status{success}");
+        $self->slog_("disconnect");
     }
+    if (! $self->{lock_error} && $self->session_lock()){
+        $self->session_lock("unlock");
+    }
+    $self->slog_("stat: $status{success}" . (($status{success}) ? " retrieved=$status{retrieved} deleted=$status{deleted} auth=$status{auth_type} creds=$status{credentials} tls=$status{tls_used} cipher=$status{tls_cipher}" : ""));
 }
 
 
@@ -1301,6 +1302,8 @@ sub add_mail {
     # split header & body
     $msg =~ s/(\r\n){2,}/\|/;
     ($header, $body) = split(/\|/, $msg, 2);
+    $header = "" unless defined $header;
+    $body = "" unless defined $body;
     $header =~ s/[\r\n]+\z//;
     $header =~ s/\A[\r\n]+//;
     $body =~ s/[\r\n]+\z//;
