@@ -55,7 +55,11 @@ sub configure_hook {
     $self->{server}->{proto} = 'udp';
     $self->{server}->{type} = SOCK_DGRAM;
     $self->{server}->{user} = INetSim::Config::getConfigParameter("Default_RunAsUser");
+    $self->{server}->{user} =~ /\A(.*)\z/; # evil untaint!
+    $self->{server}->{user} = $1;
     $self->{server}->{group} = INetSim::Config::getConfigParameter("Default_RunAsGroup");
+    $self->{server}->{group} =~ /\A(.*)\z/; # evil untaint!
+    $self->{server}->{group} = $1;
 
     $self->{servicename} = INetSim::Config::getConfigParameter("TFTP_ServiceName");
     $self->{maxchilds} = INetSim::Config::getConfigParameter("Default_MaxChilds");
@@ -85,17 +89,17 @@ sub configure_hook {
         exit 1;
     }
 
-    $gid = getgrnam(INetSim::Config::getConfigParameter("Default_RunAsGroup"));
+    $gid = getgrnam($self->{server}->{group});
     if (! defined $gid) {
-        INetSim::Log::MainLog("Warning: Unable to get GID for group 'inetsim'", $self->{servicename});
+        INetSim::Log::MainLog("Warning: Unable to get GID for group '$self->{server}->{group}'", $self->{servicename});
     }
     chown -1, $gid, $self->{upload_dir};
     ($dev, $inode, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat $self->{upload_dir};
 
-    # check for group owner 'inetsim'
+    # check group owner
     $grpname = getgrgid $gid;
-    if ($grpname ne INetSim::Config::getConfigParameter("Default_RunAsGroup")) {
-        INetSim::Log::MainLog("Warning: Group owner of Upload directory '$self->{upload_dir}' is not 'inetsim' but '$grpname'", $self->{servicename});
+    if ($grpname ne $self->{server}->{group}) {
+        INetSim::Log::MainLog("Warning: Group owner of Upload directory '$self->{upload_dir}' is not '$self->{server}->{group}' but '$grpname'", $self->{servicename});
     }
     # check for group r/w permissions
     if ((($mode & 0060) >> 3) != 6) {
